@@ -21,8 +21,8 @@ use syn::{
     LitStr, Meta, MetaNameValue, NestedMeta, Path, Result, Index,
 };
 
-const NAME: &'static str = "display";
-const EXAMPLE: &'static str = r#"#[display("format {} string" | Trait | Type::function)]"#;
+const NAME: &str = "display";
+const EXAMPLE: &str = r#"#[display("format {} string" | Trait | Type::function)]"#;
 
 #[derive(Copy, Clone, PartialEq, Eq, Debug)]
 enum FormattingTrait {
@@ -47,14 +47,14 @@ impl FormattingTrait {
             )),
             |segment| {
                 Ok(match segment.ident.to_string().as_str() {
-                    "Debug" => Some(Self::Debug),
-                    "Octal" => Some(Self::Octal),
-                    "Binary" => Some(Self::Binary),
-                    "Pointer" => Some(Self::Pointer),
-                    "LowerHex" => Some(Self::LowerHex),
-                    "UpperHex" => Some(Self::UpperHex),
-                    "LowerExp" => Some(Self::LowerExp),
-                    "UpperExp" => Some(Self::UpperExp),
+                    "Debug" => Some(FormattingTrait::Debug),
+                    "Octal" => Some(FormattingTrait::Octal),
+                    "Binary" => Some(FormattingTrait::Binary),
+                    "Pointer" => Some(FormattingTrait::Pointer),
+                    "LowerHex" => Some(FormattingTrait::LowerHex),
+                    "UpperHex" => Some(FormattingTrait::UpperHex),
+                    "LowerExp" => Some(FormattingTrait::LowerExp),
+                    "UpperExp" => Some(FormattingTrait::UpperExp),
                     _ => None,
                 })
             },
@@ -132,25 +132,25 @@ impl Technique {
         {
             Some(Meta::List(list)) => {
                 if list.nested.len() > 2 {
-                    err!(span, "too many arguments")
+                    return Err(attr_err!(span, "too many arguments"));
                 }
                 let mut iter = list.nested.iter();
                 let mut res = match iter.next() {
                     Some(NestedMeta::Lit(Lit::Str(format))) => {
-                        Some(Self::WithFormat(format.clone(), None))
+                        Some(Technique::WithFormat(format.clone(), None))
                     }
                     Some(NestedMeta::Meta(Meta::Path(path))) if path.is_ident("doc_comments") => {
-                        Some(Self::DocComments(String::new()))
+                        Some(Technique::DocComments(String::new()))
                     }
                     Some(NestedMeta::Meta(Meta::Path(path))) if path.is_ident("inner") => {
-                        Some(Self::Inner)
+                        Some(Technique::Inner)
                     }
                     Some(NestedMeta::Meta(Meta::Path(path))) => Some(
                         FormattingTrait::from_path(path, list.span())?
-                            .map_or(Self::FromMethod(path.clone()), |fmt| Self::FromTrait(fmt)),
+                            .map_or(Technique::FromMethod(path.clone()), Technique::FromTrait),
                     ),
-                    Some(_) => err!(span, "argument must be a string literal"),
-                    None => err!(span, "argument is required"),
+                    Some(_) => return Err(attr_err!(span, "argument must be a string literal")),
+                    None => return Err(attr_err!(span, "argument is required")),
                 };
                 res = match iter.next() {
                     Some(NestedMeta::Meta(Meta::NameValue(MetaNameValue {
@@ -159,43 +159,49 @@ impl Technique {
                         ..
                     }))) if Some("alt".to_string()) == path.get_ident().map(Ident::to_string) => {
                         if iter.count() > 0 {
-                            err!(span, "excessive arguments")
+                            return Err(attr_err!(span, "excessive arguments"));
                         }
                         match res {
                             Some(Technique::WithFormat(fmt, _)) => {
                                 Some(Technique::WithFormat(fmt, Some(alt.clone())))
                             }
-                            _ => err!(
-                                span,
-                                "alternative formatting can be given only if \
+                            _ => {
+                                return Err(attr_err!(
+                                    span,
+                                    "alternative formatting can be given only if \
                                  the first argument is a format string"
-                            ),
+                                ))
+                            }
                         }
                     }
                     None => res,
-                    _ => err!(span, "unrecognizable second argument"),
+                    _ => return Err(attr_err!(span, "unrecognizable second argument")),
                 };
                 res
             }
             Some(Meta::NameValue(MetaNameValue {
                 lit: Lit::Str(format),
                 ..
-            })) => Some(Self::WithFormat(format, None)),
-            Some(_) => err!(span, "argument must be a string literal"),
+            })) => Some(Technique::WithFormat(format, None)),
+            Some(_) => return Err(attr_err!(span, "argument must be a string literal")),
             None => None,
         };
 
-        res.as_mut().map(|r| r.apply_docs(attrs));
-        res.as_mut().map(|r| r.fix_fmt());
+        if let Some(r) = res.as_mut() {
+            r.apply_docs(attrs)
+        }
+        if let Some(r) = res.as_mut() {
+            r.fix_fmt()
+        };
 
         Ok(res)
     }
 
     pub fn to_fmt(&self, alt: bool) -> TokenStream2 {
         match self {
-            Self::FromTrait(fmt) => fmt.to_fmt(alt),
-            Self::FromMethod(_) => quote! { "{}" },
-            Self::WithFormat(fmt, fmt_alt) => {
+            Technique::FromTrait(fmt) => fmt.to_fmt(alt),
+            Technique::FromMethod(_) => quote! { "{}" },
+            Technique::WithFormat(fmt, fmt_alt) => {
                 if alt && fmt_alt.is_some() {
                     let alt = fmt_alt
                         .as_ref()
@@ -205,8 +211,8 @@ impl Technique {
                     quote! {#fmt}
                 }
             }
-            Self::DocComments(doc) => quote! { #doc },
-            Self::Inner => {
+            Technique::DocComments(doc) => quote! { #doc },
+            Technique::Inner => {
                 if alt {
                     quote! { "{_0:#}" }
                 } else {
@@ -297,7 +303,7 @@ impl Technique {
     }
 
     fn fix_fmt(&mut self) {
-        fn fix(s: &String) -> String {
+        fn fix(s: &str) -> String {
             s.replace("{0", "{_0")
                 .replace("{1", "{_1")
                 .replace("{2", "{_2")
@@ -310,20 +316,20 @@ impl Technique {
                 .replace("{9", "{_9")
         }
 
-        if let Self::WithFormat(fmt, x) = self {
-            *self = Self::WithFormat(
+        if let Technique::WithFormat(fmt, x) = self {
+            *self = Technique::WithFormat(
                 LitStr::new(&fix(&fmt.value()), Span::call_site()),
                 x.clone(),
             );
         }
-        if let Self::WithFormat(x, Some(fmt)) = self {
-            *self = Self::WithFormat(
+        if let Technique::WithFormat(x, Some(fmt)) = self {
+            *self = Technique::WithFormat(
                 x.clone(),
                 Some(LitStr::new(&fix(&fmt.value()), Span::call_site())),
             );
         }
-        if let Self::DocComments(fmt) = self {
-            *self = Self::DocComments(fix(fmt))
+        if let Technique::DocComments(fmt) = self {
+            *self = Technique::DocComments(fix(fmt))
         }
     }
 }
@@ -346,13 +352,15 @@ fn inner_struct(input: &DeriveInput, data: &DataStruct) -> Result<TokenStream2>
     let (impl_generics, ty_generics, where_clause) = input.generics.split_for_impl();
     let ident_name = &input.ident;
 
-    let mut technique = Technique::from_attrs(&input.attrs, input.span())?.ok_or(Error::new(
-        input.span(),
-        format!(
-            "Deriving `Display`: required attribute `{}` is missing.\n{}",
-            NAME, EXAMPLE
-        ),
-    ))?;
+    let mut technique = Technique::from_attrs(&input.attrs, input.span())?.ok_or_else(|| {
+        Error::new(
+            input.span(),
+            format!(
+                "Deriving `Display`: required attribute `{}` is missing.\n{}",
+                NAME, EXAMPLE
+            ),
+        )
+    })?;
     technique.apply_docs(&input.attrs);
 
     let tokens_fmt = technique.to_fmt(false);
@@ -361,27 +369,15 @@ fn inner_struct(input: &DeriveInput, data: &DataStruct) -> Result<TokenStream2>
     let str_alt = tokens_alt.to_string();
 
     let display = match (&data.fields, &technique) {
-        (_, Technique::FromTrait(_)) | (_, Technique::FromMethod(_)) => {
-            let a = technique
-                .clone()
-                .into_token_stream2(&data.fields, input.span(), false);
-            let b = technique
-                .clone()
-                .into_token_stream2(&data.fields, input.span(), true);
-            quote_spanned! { input.span() =>
-                if !f.alternate() {
-                    #a
-                } else {
-                    #b
-                }
-            }
-        }
+        (_, Technique::FromTrait(_)) | (_, Technique::FromMethod(_)) => technique
+            .clone()
+            .into_token_stream2(&data.fields, input.span(), false),
         (Fields::Named(fields), Technique::Inner) => {
             if fields.named.len() != 1 {
-                err!(
+                return Err(attr_err!(
                     fields.span(),
                     "display(inner) requires only single field in the structure"
-                );
+                ));
             }
             let field = fields
                 .named
@@ -391,11 +387,7 @@ fn inner_struct(input: &DeriveInput, data: &DataStruct) -> Result<TokenStream2>
                 .as_ref()
                 .expect("named fields always have ident with the name");
             quote_spanned! { field.span() =>
-                if !f.alternate() {
-                    write!(f, #tokens_fmt, _0 = self.#field)
-                } else {
-                    write!(f, #tokens_alt, _0 = self.#field)
-                }
+                write!(f, #tokens_fmt, _0 = self.#field)
             }
         }
         (Fields::Named(fields), _) => {
@@ -404,19 +396,25 @@ fn inner_struct(input: &DeriveInput, data: &DataStruct) -> Result<TokenStream2>
                 .clone()
                 .filter(|ident| has_formatters(ident, &str_fmt))
                 .collect::<Vec<_>>();
-            let idents_alt = f
-                .filter(|ident| has_formatters(ident, &str_alt))
-                .collect::<Vec<_>>();
-            quote_spanned! { fields.span() =>
-                if !f.alternate() {
+            if str_fmt == str_alt {
+                quote_spanned! { fields.span() =>
                     write!(f, #tokens_fmt, #( #idents = self.#idents, )*)
-                } else {
-                    write!(f, #tokens_alt, #( #idents_alt = self.#idents_alt, )*)
+                }
+            } else {
+                let idents_alt = f
+                    .filter(|ident| has_formatters(ident, &str_alt))
+                    .collect::<Vec<_>>();
+                quote_spanned! { fields.span() =>
+                    if !f.alternate() {
+                        write!(f, #tokens_fmt, #( #idents = self.#idents, )*)
+                    } else {
+                        write!(f, #tokens_alt, #( #idents_alt = self.#idents_alt, )*)
+                    }
                 }
             }
         }
         (Fields::Unnamed(fields), _) => {
-            let f = (0..fields.unnamed.len()).map(|i| Index::from(i));
+            let f = (0..fields.unnamed.len()).map(Index::from);
             let idents = f
                 .clone()
                 .filter(|ident| has_formatters(format!("_{}", ident.index), &str_fmt));
@@ -425,24 +423,36 @@ fn inner_struct(input: &DeriveInput, data: &DataStruct) -> Result<TokenStream2>
                 .map(|ident| Ident::new(&format!("_{}", ident.index), fields.span()))
                 .collect::<Vec<_>>();
             let idents = idents.collect::<Vec<_>>();
-            let idents_alt =
-                f.filter(|ident| has_formatters(format!("_{}", ident.index), &str_alt));
-            let nums_alt = idents_alt
-                .clone()
-                .map(|ident| Ident::new(&format!("_{}", ident.index), fields.span()))
-                .collect::<Vec<_>>();
-            let idents_alt = idents_alt.collect::<Vec<_>>();
-            quote_spanned! { fields.span() =>
-                if !f.alternate() {
+            if str_fmt == str_alt {
+                quote_spanned! { fields.span() =>
                     write!(f, #tokens_fmt, #( #nums = self.#idents, )*)
-                } else {
-                    write!(f, #tokens_alt, #( #nums_alt = self.#idents_alt, )*)
+                }
+            } else {
+                let idents_alt =
+                    f.filter(|ident| has_formatters(format!("_{}", ident.index), &str_alt));
+                let nums_alt = idents_alt
+                    .clone()
+                    .map(|ident| Ident::new(&format!("_{}", ident.index), fields.span()))
+                    .collect::<Vec<_>>();
+                let idents_alt = idents_alt.collect::<Vec<_>>();
+                quote_spanned! { fields.span() =>
+                    if !f.alternate() {
+                        write!(f, #tokens_fmt, #( #nums = self.#idents, )*)
+                    } else {
+                        write!(f, #tokens_alt, #( #nums_alt = self.#idents_alt, )*)
+                    }
                 }
             }
         }
         (Fields::Unit, _) => {
-            quote_spanned! { data.fields.span() =>
-                f.write_str(if !f.alternate() { #tokens_fmt } else { #tokens_alt })
+            if str_fmt == str_alt {
+                quote_spanned! { data.fields.span() =>
+                    f.write_str(#tokens_fmt)
+                }
+            } else {
+                quote_spanned! { data.fields.span() =>
+                    f.write_str(if !f.alternate() { #tokens_fmt } else { #tokens_alt })
+                }
             }
         }
     };
@@ -462,10 +472,11 @@ fn inner_enum(input: &DeriveInput, data: &DataEnum) -> Result<TokenStream2> {
     let mut display = TokenStream2::new();
 
     let global = Technique::from_attrs(&input.attrs, input.span())?;
-    let mut use_global = if let Some(Technique::Inner) = global {
-        false
-    } else {
-        true
+    // Ancient rust versions do not known about `matches!` macro
+    #[allow(clippy::match_like_matches_macro)]
+    let mut use_global = match global {
+        Some(Technique::Inner) => false,
+        _ => true,
     };
 
     for v in &data.variants {
@@ -474,7 +485,7 @@ fn inner_enum(input: &DeriveInput, data: &DataEnum) -> Result<TokenStream2> {
 
         let mut local = Technique::from_attrs(&v.attrs, v.span())?;
         let mut parent = global.clone();
-        let current = local.as_mut().or(parent.as_mut());
+        let current = local.as_mut().or_else(|| parent.as_mut());
         let mut current = current
             .map(|r| {
                 r.apply_docs(&v.attrs);
@@ -488,11 +499,11 @@ fn inner_enum(input: &DeriveInput, data: &DataEnum) -> Result<TokenStream2> {
 
         if let Some(Technique::DocComments(_)) = current {
             use_global = false;
-            current.as_mut().map(|t| {
+            if let Some(t) = current.as_mut() {
                 *t = Technique::DocComments(String::new());
                 t.apply_docs(&v.attrs);
                 t.fix_fmt();
-            });
+            }
         }
 
         let tokens_fmt = current.as_ref().map(|t| t.to_fmt(false));
@@ -517,10 +528,10 @@ fn inner_enum(input: &DeriveInput, data: &DataEnum) -> Result<TokenStream2> {
             (Fields::Named(fields), Some(tokens_fmt), Some(tokens_alt)) => {
                 if let Some(Technique::Inner) = current {
                     if fields.named.len() != 1 {
-                        err!(
+                        return Err(attr_err!(
                             fields.span(),
                             "display(inner) requires only single field in the structure"
-                        );
+                        ));
                     }
                     let field = fields
                         .named
@@ -531,23 +542,15 @@ fn inner_enum(input: &DeriveInput, data: &DataEnum) -> Result<TokenStream2> {
                         .expect("named fields always have ident with the name");
                     display.extend(quote_spanned! { v.span() =>
                         Self::#type_name { #field, .. } => {
-                            if !f.alternate() {
-                                write!(f, #tokens_fmt, _0 = #field)
-                            } else {
-                                write!(f, #tokens_alt, _0 = #field)
-                            }
+                            write!(f, #tokens_fmt, _0 = #field)
                         }
                     });
                 } else if let Some(Technique::FromTrait(tr)) = current {
-                    let a = Technique::FromTrait(tr).into_token_stream2(&v.fields, v.span(), false);
-                    let b = Technique::FromTrait(tr).into_token_stream2(&v.fields, v.span(), true);
+                    let stream =
+                        Technique::FromTrait(tr).into_token_stream2(&v.fields, v.span(), false);
                     display.extend(quote_spanned! { v.span() =>
                         Self::#type_name { .. } => {
-                            if !f.alternate() {
-                                #a
-                            } else {
-                                #b
-                            }
+                            #stream
                         }
                     })
                 } else {
@@ -572,15 +575,11 @@ fn inner_enum(input: &DeriveInput, data: &DataEnum) -> Result<TokenStream2> {
             }
             (Fields::Unnamed(fields), Some(tokens_fmt), Some(tokens_alt)) => {
                 if let Some(Technique::FromTrait(tr)) = current {
-                    let a = Technique::FromTrait(tr).into_token_stream2(&v.fields, v.span(), false);
-                    let b = Technique::FromTrait(tr).into_token_stream2(&v.fields, v.span(), true);
+                    let stream =
+                        Technique::FromTrait(tr).into_token_stream2(&v.fields, v.span(), false);
                     display.extend(quote_spanned! { v.span() =>
                         Self::#type_name(..) => {
-                            if !f.alternate() {
-                                #a
-                            } else {
-                                #b
-                            }
+                            #stream
                         }
                     })
                 } else {
@@ -644,6 +643,7 @@ fn inner_enum(input: &DeriveInput, data: &DataEnum) -> Result<TokenStream2> {
 
     Ok(quote! {
         impl #impl_generics ::std::fmt::Display for #ident_name #ty_generics #where_clause {
+            #![allow(clippy::if_same_then_else)]
             fn fmt(&self, mut f: &mut ::std::fmt::Formatter<'_>) -> ::std::fmt::Result {
                 #content
             }
@@ -666,7 +666,7 @@ fn inner_union(input: &DeriveInput, data: &DataUnion) -> Result<TokenStream2> {
         let type_str = format!("{}", type_name);
 
         let format = Technique::from_attrs(&field.attrs, field.span())?
-            .or(global.clone())
+            .or_else(|| global.clone())
             .map(|t| (t.to_fmt(false), t.to_fmt(true)));
 
         match format {
@@ -706,6 +706,7 @@ fn inner_union(input: &DeriveInput, data: &DataUnion) -> Result<TokenStream2> {
     };
     Ok(quote! {
         impl #impl_generics ::std::fmt::Display for #ident_name #ty_generics #where_clause {
+            #![allow(clippy::if_same_then_else)]
             fn fmt(&self, mut f: &mut ::std::fmt::Formatter<'_>) -> ::std::fmt::Result {
                 #content
             }
